<?php
/*	*********************************************************************
	*
	* Copyright (c) 2009
	* Nicolas Germain
	* http://www.fotoremix.net/
	*
	* fotoremix : plugin spip dde traitement d'images
	*
	* Ce programme est un logiciel libre distribue sous licence GNU/GPL.
	*
	*********************************************************************	*/



function image_pastille($im, $shape='circle', $color='ff0000', $v='left', $h='top')
	{
	// SPIP
	$image = image_valeurs_trans($im, "pastille-$shape-$color-$v-$h");
	
	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on definit une couleur d'ecriture
		$colordec_ = couleur_hex_to_dec($color);
		$acolor = imagecolorallocate($im_, $colordec_['red'], $colordec_['green'], $colordec_['blue']);
		
		// taille fixe de la pastille en pixels
		//$size = 13;	// bonne valeur par defaut
		$size = round(0.07 * min($x_i,$y_i));
		
		// on remplit l'image selon la forme
		
		switch ($shape)
			{
			case 'square' :
				// estimation de la position en fonction de $h et $v
				if ($v == 'left') $dest_x = $size;
				else $dest_x = $x_i - 2 * $size;
				
				if ($h == 'top') $dest_y = $size;
				else $dest_y = $y_i - 2 * $size;

				imagefilledrectangle($im_, $dest_x, $dest_y, $dest_x + $size, $dest_y + $size, $acolor);
				break;

			case 'star' :
				// estimation de la position en fonction de $h et $v
				if ($v == 'left') $x = 1.5 * $size;
				else $x = $x_i - 1.5 * $size;
				
				if ($h == 'top') $y = 1.5 * $size;
				else $y = $y_i - 1.5 * $size;
				
				// fabrication du polygone
				
				$s2 = round($size/2);
				$s4 = floor($size/4) - 1;
				$values = array(
					$x, $y+$s2, //Point1(x,y)
					$x+$s4, $y+$s4, //Point2(x,y)
					$x+$s2, $y, //Point3(x,y)
					$x+$s4, $y-$s4, //Point4(x,y)
					$x, $y-$s2, //Point5(x,y)
					$x-$s4, $y-$s4, //Point6(x,y)
					$x-$s2, $y, //Point7(x,y)
					$x-$s4, $y+$s4, //Point8(x,y)
		);

				imagefilledpolygon($im_, $values, floor(count($values)/2), $acolor);
				break;			
			
			case 'circle' :
			default :
				// estimation de la position en fonction de $h et $v
				if ($v == 'left') $dest_x = 1.5 * $size;
				else $dest_x = $x_i - 1.5 * $size;
				
				if ($h == 'top') $dest_y = 1.5 * $size;
				else $dest_y = $y_i - 1.5 * $size;
		
				imagefilledellipse($im_, $dest_x, $dest_y, $size, $size, $acolor);
				break;
			}

		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_decore($im, $masque, $v='left', $h='top')
	{
	// SPIP
	$image = image_valeurs_trans($im, "decore-$masque-$v-$h");
	
	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// init
		$enfoncement = 2/3;
		
		// on definit l'image de masque
		$im_masque = @imagecreatefrompng($masque);
		//imagealphablending($im_masque, false);
		//imagesavealpha($im_masque, true);
		$x_masque = imagesx($im_masque) - round(imagesx($im_masque)*$enfoncement);
		$y_masque = imagesy($im_masque) - round(imagesy($im_masque)*$enfoncement);
		
		// on definit les images sources et destination
		// l'image de destination est plus grande de la moitie du masque
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = @imagecreatetruecolor($x_i + $x_masque, $y_i + $y_masque);

		// reglage des options de transparence
		imagealphablending($im_, false);
		imagesavealpha($im_, true);
		
		// on remplit l'image avec la couleur transparente
		imagefilledrectangle($im_, 0, 0, $x_i + $x_masque, $y_i + $y_masque, imagecolorallocatealpha($im_, 255, 255, 255, 127));

		// estimation du decalage en fonction de $h et $v
		if ($v == 'left')
			{
			$dest_x = $x_masque;
			$dest_x_masque = 0;
			}
		else
			{
			$dest_x = 0;
			$dest_x_masque = imagesx($im_) - imagesx($im_masque);
			}
		
		if ($h == 'top')
			{
			$dest_y = $y_masque;
			$dest_y_masque = 0;
			}
		else
			{
			$dest_y = 0;
			$dest_y_masque = imagesy($im_) - imagesy($im_masque);
			}
		
		// on copie l'image source
		imagecopymerge($im_, $im, $dest_x, $dest_y, 0, 0, $x_i, $y_i, 100);
		
		// on copie le masque
		imagealphablending($im_, true);
		imagecopy($im_, $im_masque, $dest_x_masque, $dest_y_masque, 0, 0, imagesx($im_masque), imagesy($im_masque));
		
		// liberation memoire
		imagedestroy($im_masque);
		
		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_copyright($im, $text, $color='808080', $size=5)
	{
	// probleme avec certains caracteres, revenir sur ce point
	$text = utf8_decode($text);
	//$text = iconv("UTF-8", "CP1252", $text);
	
	// SPIP
	$image = image_valeurs_trans($im, "copyright-$text-$color-$size");
	
	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on definit une couleur d'ecriture
		$colordec_ = couleur_hex_to_dec($color);
		$acolor = imagecolorallocate($im_, $colordec_['red'], $colordec_['green'], $colordec_['blue']);
		$bcolor = imagecolorallocate($im_, 0, 0, 0);

		// on evalue la taille du texte et lecartement associe pour centrer le texte
		if ($size > 5) $size=5;
		else if ($size < 2) $size=1;
		
		switch (floor($size))
			{
			case 1 :
				$delta = 2.5;
				break;

			case 2 :
				$delta = 3;
				break;

			case 3 :
				$delta = 3.5;
				break;

			case 4 :
				$delta = 4;
				break;

			default :
				$delta = 4.5;
				break;
			}

		// positionne le texte au centre de l'image
		$position_x = max(0,$x_i/2 - $delta*strlen($text));
		$position_y = round($y_i/2);
		
		
		// dessine l'ombre du texte dans l'image
		imagestring($im_, $size, $position_x + 1, $position_y + 1, $text, $bcolor);

		// dessine le texte dans l'image
		imagestring($im_, $size, $position_x, $position_y, $text, $acolor);

		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_pixelate($im, $size=5)
	{
	if (!function_exists('imagefilter') || !is_long(IMG_FILTER_PIXELATE)) return '';
	
	// SPIP
	$image = image_valeurs_trans($im, "pixelate-$size-");

	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on pixelise l'image
		imagefilter($im_, IMG_FILTER_PIXELATE, $size);
		
		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_emboss($im)
	{
	if (!function_exists('imagefilter') || !is_long(IMG_FILTER_EMBOSS)) return '';
	
	// SPIP
	$image = image_valeurs_trans($im, "emboss-");

	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on donne l'effet de gravage
		imagefilter($im_, IMG_FILTER_EMBOSS);
		
		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_gaussianblur($im)
	{
	if (!function_exists('imagefilter') || !is_long(IMG_FILTER_GAUSSIAN_BLUR)) return '';
	
	// SPIP
	$image = image_valeurs_trans($im, "gaussianblur-");

	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on donne l'effet blur gausseen
		imagefilter($im_, IMG_FILTER_GAUSSIAN_BLUR);
		
		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_edgedetect($im)
	{
	if (!function_exists('imagefilter') || !is_long(IMG_FILTER_EDGEDETECT)) return '';
	
	// SPIP
	$image = image_valeurs_trans($im, "edgedetect-");

	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on donne l'effet de bord
		imagefilter($im_, IMG_FILTER_EDGEDETECT);
		
		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_colorize($im, $r=150, $v=45, $b=90)
	{
	if (!function_exists('imagefilter') || !is_long(IMG_FILTER_COLORIZE)) return '';
	
	// SPIP
	$image = image_valeurs_trans($im, "colorize-$r-$v-$b");

	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on donne l'effet de colorisation
		imagefilter($im_, IMG_FILTER_COLORIZE, $r, $v, $b);
		
		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_negate($im)
	{
	if (!function_exists('imagefilter') || !is_long(IMG_FILTER_NEGATE)) return '';
	
	// SPIP
	$image = image_valeurs_trans($im, "negate-");

	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on donne l'effet negatif
		imagefilter($im_, IMG_FILTER_NEGATE);
		
		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_smooth($im, $val=0)
	{
	if (!function_exists('imagefilter') || !is_long(IMG_FILTER_SMOOTH)) return '';
	
	// SPIP
	$image = image_valeurs_trans($im, "smooth-$val");

	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on adoucit l'image
		imagefilter($im_, IMG_FILTER_SMOOTH, $val);
		
		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_brightness($im, $val=0)
	{
	if (!function_exists('imagefilter') || !is_long(IMG_FILTER_BRIGHTNESS)) return '';
	
	// SPIP
	$image = image_valeurs_trans($im, "smooth-$val");

	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on modifie la luminosite de l'image
		imagefilter($im_, IMG_FILTER_BRIGHTNESS, $val);
		
		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_meanremoval($im)
	{
	if (!function_exists('imagefilter') || !is_long(IMG_FILTER_MEAN_REMOVAL)) return '';
	
	// SPIP
	$image = image_valeurs_trans($im, "bruit-");

	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on donne du bruit sur l'image
		imagefilter($im_, IMG_FILTER_MEAN_REMOVAL);
		
		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_contrast($im, $val=0)
	{
	if (!function_exists('imagefilter') || !is_long(IMG_FILTER_CONTRAST)) return '';
	
	// SPIP
	$image = image_valeurs_trans($im, "contraste-$val");

	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on change le contraste de l'image
		imagefilter($im_, IMG_FILTER_CONTRAST, $val);
		
		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_grayscale($im)
	{
	if (!function_exists('imagefilter') || !is_long(IMG_FILTER_GRAYSCALE)) return '';
	
	// SPIP
	$image = image_valeurs_trans($im, "gris-");

	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on change le contraste de l'image
		imagefilter($im_, IMG_FILTER_GRAYSCALE);
		
		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}


function image_convolution($im)
	{
	if (!function_exists('imageconvolution')) return '';

	// SPIP
	$image = image_valeurs_trans($im, "convolution-");

	if (!$image) return("");
	
	$x_i = $image["largeur"];
	$y_i = $image["hauteur"];
	
	$im = $image["fichier"];
	$dest = $image["fichier_dest"];
	$creer = $image["creer"];
	
	if ($creer)
		{
		// on definit les images sources et destination
		$im = $image["fonction_imagecreatefrom"]($im);
		$im_ = imagecreatetruecolor($x_i, $y_i);
		
		// on copie l'image source
		imagecopy($im_, $im, 0, 0, 0, 0, $x_i, $y_i);
		
		// on donne l'effet de convolution selon la matrice emboss
		$emboss = array(array(2, 0, 0), array(0, -1, 0), array(0, 0, -1));

		imageconvolution($im_, $emboss, 10, 80);
		
		// SPIP
		$image["fonction_image"]($im_, "$dest");	
		}

	// SPIP
	$class = $image["class"];
	if (strlen($class) > 1) $tags=" class='$class'";
	$tags = "$tags alt='".$image["alt"]."'";
	$style = $image["style"];
	
	return "<img src='$dest'$tags />";
	}
?>